with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure circle is

  type vec3 is array (1 .. 3) of long_float;

  maxls,maxminor,maxlist,maxmajor : integer;
  lstol,mntol,mnref               : long_float;
  n,nn,ne                         : integer;
  radius,pressure                 : long_float;
  young1,young2,poisson12,shear12 : long_float;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    return 3 * i * (i - 1) + j + 1;
  end node;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    r,t : long_float;

  begin
    r := radius / long_float(n) * long_float(i);
    x(3) := 0.0;

    if 2 * (j - 1) = 0 then
      x(1) := r;
      x(2) := 0.0;
    elsif 2 * (j - 1) = 3 * i then
      x(1) := 0.0;
      x(2) := r;
    elsif 2 * (j - 1) = 6 * i then
      x(1) := -r;
      x(2) := 0.0;
    elsif 2 * (j - 1) = 9 * i then
      x(1) := 0.0;
      x(2) := -r;
    else
      t := pi / 3.0 / long_float(i) * long_float(j - 1);
      x(1) := r * cos(t);
      x(2) := r * sin(t);
    end if;
  end vertex;

  function firstj (i,k : in integer) return integer is

  begin
    return (k - 1) * i + 1;
  end firstj;

  function nextj (i,j : in integer) return integer is

  begin
    return j mod (6 * i) + 1;
  end nextj;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "circle.txt");

    scan(io);
    get(io,n);
    get(io,radius);
    skip_line(io);

    -- young1    : Young's modulus in direction 1, multiplied by thickness
    -- young2    : Young's modulus in direction 2, multiplied by thickness
    -- poisson12 : Poissons ratio for strain in direction 2 when stressed in direction 1
    -- shear12   : Shear modulus in the 1,2 plane, multiplied by thickness

    scan(io);
    get(io,young1);
    get(io,young2);
    get(io,poisson12);
    get(io,shear12);
    skip_line(io);

    scan(io);
    get(io,pressure);
    skip_line(io);

    close(io);

    -- initiate surface
    nn := 3 * n * (n + 1) + 1;
    ne := 6 * n * n;

    -- solution parameters
    lstol := 0.1;
    maxls := 20;
    mntol := 0.001;
    mnref := 0.0;
    maxminor := 30 * nn;
    maxlist := 3;
    maxmajor := 20;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics.long_elementary_functions;

    e,j1,j2,j3,glob : integer;
    x,xi,xj         : vec3;
    norm,cost,sint  : long_float;
    io              : file_type;

  begin
    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls             mntol             mnref  maxminor   maxlist  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,mntol,2,6,3);
    put(io,"     ");
    put(io,mnref,2,6,3);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- young1, young2, poisson12, shear12, density
    new_line(io);
    new_line(io);
    put(io,"       young1            young2         poisson12           shear12           density");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,young1,2,6,3);
    put(io,"     ");
    put(io,young2,2,6,3);
    put(io,"     ");
    put(io,poisson12,2,6,3);
    put(io,"     ");
    put(io,shear12,2,6,3);
    put(io,"      0.000000E+00");
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);

    -- surface
    put(io,"    1      0.000000E+00      0.000000E+00      0.000000E+00");
    new_line(io);
    for i in 1 .. n loop
      for j in 1 .. 6 * i loop
        vertex(i,j,x);
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- membrane elements
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node            cosine              sine");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    e := 0;
    new_line(io);

    for k in 1 .. 6 loop
      j2 := firstj(1,k);
      j3 := nextj(1,j2);

      e := e + 1;
      put(io,e,5);
      put(io,"         1     ");
      put(io,node(1,j2),5);
      put(io,"     ");
      put(io,node(1,j3),5);

      -- cost, sint
      vertex(1,j2,x);
      norm := sqrt(x(1) ** 2 + x(2) ** 2);
      cost := x(1) / norm;
      sint := -x(2) / norm;
      put(io,"     ");
      put(io,cost,2,6,3);
      put(io,"     ");
      put(io,sint,2,6,3);
      new_line(io);

      for i in 1 .. n - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        for s in 1 .. i + 1 loop
          j3 := nextj(i + 1,j2);

          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i,j1),5);
          put(io,"     ");
          put(io,node(i + 1,j2),5);
          put(io,"     ");
          put(io,node(i + 1,j3),5);

          -- cost, sint
          vertex(i + 1,j2,xj);
          vertex(i,j1,xi);
          x(1) := xj(1) - xi(1);
          x(2) := xj(2) - xi(2);
          norm := sqrt(x(1) ** 2 + x(2) ** 2);
          cost := x(1) / norm;
          sint := -x(2) / norm;
          put(io,"     ");
          put(io,cost,2,6,3);
          put(io,"     ");
          put(io,sint,2,6,3);
          new_line(io);

          j1 := nextj(i,j1);
          j2 := j3;
        end loop;
      end loop;
    end loop;

    for k in 1 .. 6 loop
      for i in 1 .. n - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        j2 := nextj(i + 1,j2);
        for s in 1 .. i loop
          j3 := nextj(i,j1);

          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i,j1),5);
          put(io,"     ");
          put(io,node(i + 1,j2),5);
          put(io,"     ");
          put(io,node(i,j3),5);

          -- cost, sint
          vertex(i + 1,j2,xj);
          vertex(i,j1,xi);
          x(1) := xj(1) - xi(1);
          x(2) := xj(2) - xi(2);
          norm := sqrt(x(1) ** 2 + x(2) ** 2);
          cost := x(1) / norm;
          sint := -x(2) / norm;
          put(io,"     ");
          put(io,cost,2,6,3);
          put(io,"     ");
          put(io,sint,2,6,3);
          new_line(io);

          j1 := j3;
          j2 := nextj(i + 1,j2);
        end loop;
      end loop;
    end loop;

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,18 * n,5);
    new_line(io);

    new_line(io);
    for j in 1 .. 6 * n loop
      glob := node(n,j);
      for axis in 1 .. 3 loop
        put(io,glob,5);
        put(io,"     ");
        put(io,axis,5);
        put(io,"      0.000000E+00");
        new_line(io);
      end loop;
    end loop;

    -- nodal actions
    new_line(io);
    new_line(io);
    put(io," node      axis            action");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- pressure
    new_line(io);
    new_line(io);
    put(io," elem          pressure");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    new_line(io);
    for e in 1 .. ne loop
      put(io,e,5);
      put(io,"     ");
      put(io,pressure,2,6,3);
      new_line(io);
    end loop;

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.long_float_text_io;

    j1,j2,j3 : integer;
    x        : vec3;
    io       : file_type;

  begin
    create(io, out_file, "circle.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- surface
    put(io,"layer make surface ");
    new_line(io);

    for k in 1 .. 6 loop
      j2 := firstj(1,k);
      j3 := nextj(1,j2);

      put(io,"3dface ");

      x(1) := 0.0;
      x(2) := 0.0;
      x(3) := 0.0;
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(1,j2,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(1,j3,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      for i in 1 .. n - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        for s in 1 .. i + 1 loop
          j3 := nextj(i + 1,j2);

          put(io,"3dface ");

          vertex(i,j1,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j2,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j3,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          put(io," ");
          new_line(io);

          j1 := nextj(i,j1);
          j2 := j3;
        end loop;
      end loop;
    end loop;

    for k in 1 .. 6 loop
      for i in 1 .. n - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        j2 := nextj(i + 1,j2);
        for s in 1 .. i loop
          j3 := nextj(i,j1);

          put(io,"3dface ");

          vertex(i,j1,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j2,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i,j3,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          put(io," ");
          new_line(io);

          j1 := j3;
          j2 := nextj(i + 1,j2);
        end loop;
      end loop;
    end loop;

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

  procedure ansys is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    e,j1,j2,j3 : integer;
    x          : vec3;
    io         : file_type;

  begin
    create(io, out_file, "ansys.txt");

    put(io,"! ENTER THE MODEL CREATION PREPROCESSOR");
    new_line(io);
    put(io,"/PREP7");
    new_line(io);

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    -- define material property
    new_line(io);
    put(io,"! DEFINE MATERIAL PROPERTY");
    new_line(io);

    put(io,"MP,   EX, 1, ");
    put(io,young1,2,6,3);
    new_line(io);

    put(io,"MP,   EY, 1, ");
    put(io,young2,2,6,3);
    new_line(io);

    put(io,"MP, PRXY, 1, ");
    put(io,poisson12,2,6,3);
    new_line(io);

    put(io,"MP,  GXY, 1, ");
    put(io,shear12,2,6,3);
    new_line(io);

    put(io,"MP, ALPX, 1,  1.000000E+00");
    new_line(io);

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    -- define nodes
    new_line(io);
    put(io,"! DEFINE NODES");
    new_line(io);

    put(io,"N,     1,  0.000000E+00,  0.000000E+00,  0.000000E+00");
    new_line(io);
    for i in 1 .. n loop
      for j in 1 .. 6 * i loop
        vertex(i,j,x);
        put(io,"N, ");
        put(io,node(i,j),5);
        put(io,", ");
        put(io,x(1),2,6,3);
        put(io,", ");
        put(io,x(2),2,6,3);
        put(io,", ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    -- define element type
    new_line(io);
    put(io,"! DEFINE ELEMENT TYPE");
    new_line(io);
    put(io,"ET, 1, SHELL41");
    new_line(io);

    -- define element real constants
    new_line(io);
    put(io,"! DEFINE ELEMENT REAL CONSTANTS");
    new_line(io);
    put(io,"R, 1, 1.0");
    new_line(io);

    -- define elements
    new_line(io);
    put(io,"! DEFINE ELEMENTS");
    new_line(io);

    e := 0;

    for k in 1 .. 6 loop
      j2 := firstj(1,k);
      j3 := nextj(1,j2);

      e := e + 1;

      put(io,"EN, ");
      put(io,e,5);
      put(io,",     1");
      put(io,", ");
      put(io,node(1,j2),5);
      put(io,", ");
      put(io,node(1,j3),5);
      put(io,", ");
      put(io,node(1,j3),5);
      new_line(io);

      for i in 1 .. n - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        for s in 1 .. i + 1 loop
          j3 := nextj(i + 1,j2);

          e := e + 1;

          put(io,"EN, ");
          put(io,e,5);
          put(io,", ");
          put(io,node(i,j1),5);
          put(io,", ");
          put(io,node(i + 1,j2),5);
          put(io,", ");
          put(io,node(i + 1,j3),5);
          put(io,", ");
          put(io,node(i + 1,j3),5);
          new_line(io);

          j1 := nextj(i,j1);
          j2 := j3;
        end loop;
      end loop;
    end loop;

    for k in 1 .. 6 loop
      for i in 1 .. n - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        j2 := nextj(i + 1,j2);
        for s in 1 .. i loop
          j3 := nextj(i,j1);

          e := e + 1;

          put(io,"EN, ");
          put(io,e,5);
          put(io,", ");
          put(io,node(i,j1),5);
          put(io,", ");
          put(io,node(i + 1,j2),5);
          put(io,", ");
          put(io,node(i,j3),5);
          put(io,", ");
          put(io,node(i,j3),5);
          new_line(io);

          j1 := j3;
          j2 := nextj(i + 1,j2);
        end loop;
      end loop;
    end loop;

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    -- include large deformation effects
    new_line(io);
    put(io,"! INCLUDE LARGE DEFORMATION EFFECTS");
    new_line(io);
    put(io,"NLGEOM, ON");
    new_line(io);

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    -- set convergence values for nonlinear analysis
    new_line(io);
    put(io,"! SET CONVERGENCE VALUES FOR NONLINEAR ANALYSIS");
    new_line(io);
    put(io,"CNVTOL, F, , ");
    put(io,mntol,2,6,3);
    put(io,", 0, -1.0");
    new_line(io);

    new_line(io);
    put(io,"FINISH");
    new_line(io);

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    new_line(io);
    put(io,"! ENTER THE SOLUTION PROCESSOR");
    new_line(io);
    put(io,"/SOLU");
    new_line(io);

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    new_line(io);
    put(io,"! DEFINE DOF CONSTRAINTS");
    new_line(io);
    put(io,"D, ALL, ALL");
    new_line(io);

    new_line(io);
    put(io,"! ASSIGN A UNIFORM TEMPERATURE TO ALL NODES");
    new_line(io);
    put(io,"TUNIF, -1.000000E-03");
    new_line(io);

    new_line(io);
    put(io,"! START SOLUTION");
    new_line(io);
    put(io,"SOLVE");
    new_line(io);

    new_line(io);
    put(io,"! ---------------------------------------");
    new_line(io);

    -- known displacements
    new_line(io);
    put(io,"! DELETE DOF CONSTRAINTS");
    new_line(io);
    put(io,"DDELE, ALL, ALL");
    new_line(io);

    new_line(io);
    put(io,"! DEFINE DOF CONSTRAINTS");
    new_line(io);

    for j in 1 .. 6 * n loop
      put(io,"D, ");
      put(io,node(n,j),5);
      put(io,", ALL,  0.000000E+00");
      new_line(io);
    end loop;

    -- specify surface loads on elements
    new_line(io);
    put(io,"! SPECIFY SURFACE LOADS ON ELEMENTS");
    new_line(io);
    put(io,"SFE, ALL, 1, PRES, 0, ");
    put(io,pressure,2,6,3);
    new_line(io);

    new_line(io);
    put(io,"! START SOLUTION");
    new_line(io);
    put(io,"SOLVE");
    new_line(io);

    new_line(io);
    put(io,"FINISH");
    new_line(io);

    close(io);
  end ansys;

begin
  initia;
  input;
  autocad;
  ansys;
end circle;
